(* symboles *)
type symbol = string

(* expressions *)
type expr =
    | TInt of int			(* i *)
    | TAdd of expr * expr		(* e1+e2 *)
    | TMul of expr * expr		(* e2*e2 *)
    | TVar of symbol			(* x *)
    | TLet of symbol * expr * expr	(* let x=e1 in e2 *)
    | TApp of expr * expr	        (* e1(e2) *)
    | TFun of symbol * expr		(* fun x -> e *)
    | TUnit			        (* () *)
    | TSeq of expr * expr	        (* e1; e2 *)
    | TRef of expr			(* ref e *)
    | TAffect of expr * expr	        (* (e1 := e2) *)
    | TDeref of expr			(* !e *)

(* affichage *)
let rec print_expr f = function
  | TUnit -> Format.fprintf f "()"
  | TInt i -> Format.fprintf f "%i" i
  | TRef a -> Format.fprintf f "ref (%a)" print_expr a
  | TAdd(a,b) -> Format.fprintf f "%a + %a" print_expr a print_expr b
  | TMul(a,b) -> Format.fprintf f "(%a) * (%a)" print_expr a print_expr b
  | TDeref a -> Format.fprintf f "!(%a)" print_expr a
  | TAffect(a,b) -> Format.fprintf f "%a := %a" print_expr a print_expr b
  | TVar x -> Format.fprintf f "%s" x
  | TSeq(a,b) -> Format.fprintf f "%a;\n%a" print_expr a print_expr b
  | TLet(x,a,b) -> Format.fprintf f "let %s = %a in\n%a" x print_expr a print_expr b
  | TApp(a,b) -> Format.fprintf f "(%a %a)" print_expr a print_expr b
  | TFun(x,a) -> Format.fprintf f "(fun %s -> %a)" x print_expr a
